@@ -1,3 +1,4 @@
+/* Copyright 2007, 2008 Peter Klausler.  See COPYING for license. */
 #include "all.h"
 
 /*
@@ -8,12 +9,13 @@
 
 struct window {
 	struct view *view;
-	unsigned start; /* locus index */
+	locus_t start;
 	unsigned row, column;
 	unsigned rows, columns;
 	unsigned cursor_row, cursor_column;
 	rgba_t fgrgba, bgrgba;
-	unsigned last_dirties, last_cursor, last_mark;
+	unsigned last_dirties;
+	position_t last_cursor, last_mark;
 	rgba_t last_fgrgba, last_bgrgba;
 	struct window *next;
 };
@@ -23,10 +25,28 @@ static struct window *active_window;
 static struct display *display;
 static unsigned display_rows, display_columns;
 
+static void title(struct window *window)
+{
+	char buff[128];
+	if (window != active_window)
+		return;
+	if (!window) {
+		display_title(display, NULL);
+		return;
+	}
+	snprintf(buff, sizeof buff, "%s%s",
+		 window->view->name,
+		 window->view->text->flags & TEXT_CREATED ? "(new)" :
+		 window->view->text->flags & TEXT_RDONLY ? "(read-only)" :
+		 window->view->text->preserved !=
+		    window->view->text->dirties ? "(unsaved)" : "");
+	display_title(display, buff);
+}
+
 static struct window *activate(struct window *window)
 {
-	if ((active_window = window))
-		display_title(display, active_window->view->name);
+	active_window = window;
+	title(window);
 	return window;
 }
 
@@ -54,7 +74,7 @@ static struct window *window_create(struct view *view, struct view *after)
 	return window;
 }
 
-static int stacked(struct window *up, struct window *down)
+static Boolean_t stacked(struct window *up, struct window *down)
 {
 	return	up && down &&
 		up->column == down->column &&
@@ -62,7 +82,7 @@ static int stacked(struct window *up, struct window *down)
 		up->row + up->rows == down->row;
 }
 
-static int beside(struct window *left, struct window *right)
+static Boolean_t beside(struct window *left, struct window *right)
 {
 	return	left && right &&
 		left->row == right->row &&
@@ -70,36 +90,36 @@ static int beside(struct window *left, struct window *right)
 		left->column + left->columns == right->column;
 }
 
-static int adjacent(struct window *x, struct window *y)
+static Boolean_t adjacent(struct window *x, struct window *y)
 {
 	if (x->column + x->columns < y->column ||
 	    y->column + y->columns < x->column ||
 	    x->row + x->rows < y->row ||
 	    y->row + y->rows < x->row)
-		return 0;
+		return FALSE;
 	if (y->column != x->column + x->columns &&
 	    x->column != y->column + y->columns)
-		return 1;
+		return TRUE;
 	return y->row < x->row + x->rows &&
 	       x->row < y->row + y->rows;
 }
 
-static int window_expand_next(struct window *window)
+static Boolean_t window_expand_next(struct window *window)
 {
 	struct window *next = window->next;
 	if (!next)
-		return 0;
+		return FALSE;
 	if (stacked(window, next)) {
 		next->row = window->row;
 		next->rows += window->rows;
-		return 1;
+		return TRUE;
 	}
 	if (beside(window, next)) {
 		next->column = window->column;
 		next->columns += window->columns;
-		return 1;
+		return TRUE;
 	}
-	return 0;
+	return FALSE;
 }
 
 void window_destroy(struct window *window)
@@ -133,9 +153,16 @@ void window_destroy(struct window *window)
 		locus_destroy(window->view, window->start);
 		window->view->window = NULL;
 	}
-	if (window == active_window)
-		activate(window_list);
-	allocate(window, 0);
+	if (window == active_window) {
+		wp = window_list;
+		if (previous) {
+			wp = previous;
+			if (wp->next)
+				wp = wp->next;
+		}
+		activate(wp);
+	}
+	RELEASE(window);
 }
 
 struct window *window_raise(struct view *view)
@@ -144,7 +171,7 @@ struct window *window_raise(struct view *view)
 		display = display_init();
 	display_get_geometry(display, &display_rows, &display_columns);
 	display_erase(display, 0, 0, display_rows, display_columns,
-		      0xff, 0xff /*default color*/);
+		      DEFAULT_FGRGBA, DEFAULT_BGRGBA);
 	while (window_list && window_list->view != view)
 		window_destroy(window_list);
 	while (window_list && window_list->next)
@@ -206,7 +233,7 @@ struct window *window_below(struct view *above, struct view *view,
 	window->column = old->column;
 	window->columns = old->columns;
 	window->row = old->row + (old->rows -= (window->rows = rows));
-	return activate(window);
+	return window;
 }
 
 struct window *window_replace(struct view *old, struct view *new)
@@ -246,20 +273,22 @@ void windows_end(void)
 	}
 }
 
-static unsigned count_rows(struct window *window, unsigned start, unsigned end)
+static unsigned count_rows(struct window *window, position_t start,
+			   position_t end)
 {
 	unsigned rows = 0, bytes;
 
 	for (rows = 0; start < end; rows++, start += bytes)
-		if (!(bytes = find_row_bytes(window->view, start, 0, window->columns)))
+		if (!(bytes = find_row_bytes(window->view, start, 0,
+					     window->columns)))
 			break;
 	return rows;
 }
 
-static unsigned find_row_start(struct window *window, unsigned position,
-			       unsigned start)
+static position_t find_row_start(struct window *window, position_t position,
+				 position_t start)
 {
-	unsigned bytes;
+	size_t bytes;
 	if (position && position >= window->view->bytes)
 		position--;
 	while ((bytes = find_row_bytes(window->view, start, 0, window->columns))) {
@@ -270,28 +299,30 @@ static unsigned find_row_start(struct window *window, unsigned position,
 	return start;
 }
 
-static void new_start(struct view *view, unsigned start)
+static void new_start(struct view *view, position_t start)
 {
 	locus_set(view, view->window->start, start ? start-1 : UNSET);
 	view->window->last_dirties = ~0;
 }
 
-static unsigned screen_start(struct view *view)
+static position_t screen_start(struct view *view)
 {
-	unsigned start = locus_get(view, view->window->start);
+	position_t start = locus_get(view, view->window->start);
 	if (start++ == UNSET)
 		start = 0;
 	return start;
 }
 
-static unsigned focus(struct window *window)
+static position_t focus(struct window *window)
 {
 	struct view *view = window->view;
-	unsigned cursor = locus_get(view, CURSOR);
-	unsigned cursorrow = find_row_start(window, cursor,
-					    find_line_start(view, cursor));
-	unsigned start = screen_start(view);
-	unsigned end, above, below, at, bytes;
+	position_t cursor = locus_get(view, CURSOR);
+	position_t cursorrow = find_row_start(window, cursor,
+					      find_line_start(view, cursor));
+	position_t start = screen_start(view);
+	position_t end, at;
+	unsigned above, below;
+	size_t bytes;
 
 	start = find_row_start(window, start,
 			       find_line_start(view, start));
@@ -325,43 +356,55 @@ static unsigned focus(struct window *window)
 		start += find_row_bytes(view, start, 0, window->columns);
 
 done:	window->cursor_column = find_column(&above, view, cursorrow,
-					    cursor, 0, window->columns);
+					    cursor, 0);
 	window->cursor_row = above;
 	new_start(view, start);
 	return start;
 }
 
-static int lame_space(struct view *view, unsigned offset, unsigned look)
+static Boolean_t lame_space(struct view *view, position_t start,
+			    unsigned next_tab)
 {
-	int all_spaces_lame = look > 1 && !no_tabs;
-	while (look--) {
-		int ch = view_char(view, offset, &offset);
-		if (ch < 0 || ch == '\n' || ch == '\t')
-			return 1;
-		if (ch != ' ')
-			return 0;
-	}
-	return all_spaces_lame;
+	unsigned distance = 0;
+	position_t offset = start;
+	Unicode_t ch;
+	if (view->shell_std_in >= 0 || view->text->flags & TEXT_EDITOR)
+		return FALSE;
+	do {
+		distance++;
+		ch = view_char(view, offset, &offset);
+		if (!IS_UNICODE(ch) || ch == '\n' || ch == '\t')
+			return TRUE;
+	} while (ch == ' ');
+	return distance > next_tab &&
+	       !(view->text->flags & TEXT_NO_TABS) &&
+	       (next_tab ||
+		view_char_prior(view, start-1, NULL) == ' ');
 }
 
-static int lame_tab(struct view *view, unsigned offset)
+static Boolean_t lame_tab(struct view *view, position_t offset)
 {
-	int ch;
-	if (no_tabs)
-		return 1;
-	for (; (ch = view_char(view, offset, &offset)) >= 0 && ch != '\n';)
+	Unicode_t ch;
+	if (view->shell_std_in >= 0 || view->text->flags & TEXT_EDITOR)
+		return FALSE;
+	if (view->text->flags & TEXT_NO_TABS)
+		return TRUE;
+	for (;
+	     IS_UNICODE(ch = view_char(view, offset, &offset)) &&
+	     ch != '\n'; )
 		if (ch != ' ' && ch != '\t')
-			return 0;
-	return 1;
+			return FALSE;
+	return TRUE;
 }
 
-static unsigned paintch(struct window *window, int ch, unsigned row,
+static unsigned paintch(struct window *window, Unicode_t ch, unsigned row,
 			unsigned column,
-			unsigned at, unsigned cursor, unsigned mark,
-			unsigned *brackets)
+			position_t at, position_t cursor, position_t mark,
+			unsigned *brackets, rgba_t fgrgba)
 {
-	rgba_t fgrgba = window->fgrgba, bgrgba = window->bgrgba;
+	rgba_t bgrgba = window->bgrgba;
 	unsigned tabstop = window->view->text->tabstop;
+	const char *brack = window->view->text->brackets;
 
 	if (ch == '\n')
 		return column;
@@ -390,40 +433,42 @@ static unsigned paintch(struct window *window, int ch, unsigned row,
 		return column;
 	}
 
-	if (ch < ' ' || ch == 0x7f ||
-	    ch >= FOLD_START && ch < FOLD_END) {
+	if (ch < ' ' || ch == 0x7f || IS_FOLDED(ch)) {
 		bgrgba = 0xff000000;
 		fgrgba = 0xffffff00;
 		display_put(display, window->row + row, window->column + column++,
-			    ch >= FOLD_START ? '<' : '^', fgrgba, bgrgba);
-		if (ch >= FOLD_START)
+			    IS_FOLDED(ch) ? '<' : '^', fgrgba, bgrgba);
+		if (IS_FOLDED(ch))
 			ch = '>';
 		else if (ch == 0x7f)
 			ch = '?';
 		else
 			ch += '@';
-	} else if (ch == ' ' &&
+	} else if (ch == ' ') {
+		if (bgrgba == window->bgrgba &&
 		   lame_space(window->view, at + 1,
 			      tabstop-1 - column % tabstop))
-		bgrgba = 0xff00ff00;
-	else if ((ch == '(' || ch == '[' || ch == '{') &&
-		 (*brackets)++ & 1 ||
-		 (ch == ')' || ch == ']' || ch == '}') &&
-		 --*brackets & 1)
-		fgrgba = 0x0000ff00;
-	else if (ch < 0)
+			bgrgba = 0xff00ff00;
+	} else if (!IS_UNICODE(ch))
 		ch = ' ', bgrgba = 0xff00ff00;
+	else if (brack)
+		for (; *brack; brack += 2)
+			if (ch == brack[0] && (*brackets)++ & 1 ||
+			    ch == brack[1] && --*brackets & 1) {
+				fgrgba = 0x0000ff00;
+				break;
+			}
 
 	display_put(display, window->row + row, window->column + column++,
 		    ch, fgrgba, bgrgba);
 	return column;
 }
 
-static int needs_repainting(struct window *window)
+static Boolean_t needs_repainting(struct window *window)
 {
 	struct view *view = window->view;
-	unsigned cursor = locus_get(view, CURSOR);
-	unsigned mark = locus_get(view, MARK);
+	position_t cursor = locus_get(view, CURSOR);
+	position_t mark = locus_get(view, MARK);
 
 	if (mark == UNSET)
 		mark = cursor;
@@ -434,7 +479,7 @@ static int needs_repainting(struct window *window)
 		window->last_mark != mark;
 }
 
-static void repainted(struct window *window, unsigned cursor, unsigned mark)
+static void repainted(struct window *window, position_t cursor, position_t mark)
 {
 	window->last_dirties = window->view->text->dirties;
 	window->last_fgrgba = window->fgrgba;
@@ -445,34 +490,53 @@ static void repainted(struct window *window, unsigned cursor, unsigned mark)
 
 static void paint(struct window *window)
 {
-	unsigned at, row, next, column;
+	position_t at, next;
+	unsigned row, column;
 	struct view *view = window->view;
-	unsigned cursor = locus_get(view, CURSOR);
-	unsigned mark = locus_get(view, MARK);
+	position_t cursor = locus_get(view, CURSOR);
+	position_t mark = locus_get(view, MARK);
 	unsigned brackets = 1;
 
-	repainted(window, cursor, mark);
+	title(window);
 
 	at = focus(window);
 	for (row = 0; row < window->rows; row++) {
 
-		unsigned limit = at + find_row_bytes(view, at, 0, window->columns);
-		for (column = 0; at < limit; at = next)
-			column = paintch(window, view_char(view, at, &next),
-					 row, column, at, cursor, mark,
-					 &brackets);
+		position_t limit = at + find_row_bytes(view, at, 0,
+						       window->columns);
+		rgba_t fgrgba = window->fgrgba;
+		Boolean_t keywords = view->text->keywords &&
+				     (fgrgba == DEFAULT_FGRGBA ||
+				      !fgrgba);
+		Boolean_t look_for_keyword = keywords;
+
+		for (column = 0; at < limit; at = next) {
+			Unicode_t ch = view_char(view, at, &next);
+			if (!is_idch(ch) && ch != '#') {
+				fgrgba = window->fgrgba;
+				look_for_keyword = keywords;
+			} else if (look_for_keyword && is_keyword(view, at))
+				fgrgba = 0x0000ff00;
+			else
+				look_for_keyword = FALSE;
+			column = paintch(window, ch, row, column, at,
+					 cursor, mark, &brackets,
+					 fgrgba);
+		}
 
 		display_erase(display, window->row + row,
 			      window->column + column, 1,
 			      window->columns - column,
 			      window->fgrgba, window->bgrgba);
 	}
+
+	repainted(window, cursor, mark);
 }
 
-void window_hint_deleting(struct window *window, unsigned offset, unsigned bytes)
+void window_hint_deleting(struct window *window, position_t offset, size_t bytes)
 {
 	struct view *view = window->view;
-	unsigned at = screen_start(view);
+	position_t at = screen_start(view);
 	unsigned row = 0, column;
 	unsigned lines = 0, end_column;
 
@@ -484,11 +548,11 @@ void window_hint_deleting(struct window *window, unsigned offset, unsigned bytes
 	}
 	if (!bytes || view_char(view, offset, NULL) >= FOLD_START)
 		return;
-	column = find_column(&row, view, at, offset, 0, window->columns);
+	column = find_column(&row, view, at, offset, 0);
 	if (row >= window->rows)
 		return;
 	end_column = find_column(&lines, view, offset,
-				 offset + bytes, column, window->columns);
+				 offset + bytes, column);
 	if (!lines) {
 		unsigned old = find_row_bytes(view, offset,
 					      column, window->columns);
@@ -513,12 +577,12 @@ void window_hint_deleting(struct window *window, unsigned offset, unsigned bytes
 			     window->fgrgba, window->bgrgba);
 }
 
-void window_hint_inserted(struct window *window, unsigned offset, unsigned bytes)
+void window_hint_inserted(struct window *window, position_t offset, size_t bytes)
 {
 	struct view *view = window->view;
-	unsigned at = screen_start(view);
-	unsigned row = 0, column;
-	unsigned lines = 0, end_column;
+	position_t at = screen_start(view);
+	position_t column, end_column;
+	unsigned row = 0, lines = 0;
 
 	if (focus(window) != at || offset + bytes <= at)
 		return;
@@ -528,16 +592,16 @@ void window_hint_inserted(struct window *window, unsigned offset, unsigned bytes
 	}
 	if (!bytes || view_char(view, offset, NULL) >= FOLD_START)
 		return;
-	column = find_column(&row, view, at, offset, 0, window->columns);
+	column = find_column(&row, view, at, offset, 0);
 	if (row >= window->rows)
 		return;
 	end_column = find_column(&lines, view, offset,
-				 offset + bytes, column, window->columns);
+				 offset + bytes, column);
 	if (!lines) {
-		unsigned old = find_row_bytes(view, offset + bytes,
-					      end_column, window->columns);
-		unsigned new = find_row_bytes(view, offset,
-					      column, window->columns);
+		size_t old = find_row_bytes(view, offset + bytes,
+					    end_column, window->columns);
+		size_t new = find_row_bytes(view, offset,
+					    column, window->columns);
 		if (new == old + bytes) {
 			display_insert_spaces(display, window->row + row,
 					      window->column + column,
@@ -566,8 +630,9 @@ void window_next(struct view *view)
 struct window *window_recenter(struct view *view)
 {
 	struct window *window = view->window;
-	unsigned cursor = locus_get(view, CURSOR);
-	unsigned start, row, end;
+	position_t cursor = locus_get(view, CURSOR);
+	position_t start, end;
+	int row;
 
 	if (!window)
 		window = window_raise(view);
@@ -586,8 +651,10 @@ struct window *window_recenter(struct view *view)
 void window_page_up(struct view *view)
 {
 	struct window *window = view->window;
-	unsigned start = screen_start(view);
-	unsigned end, row, bytes;
+	position_t start = screen_start(view);
+	position_t end;
+	int row;
+	size_t bytes;
 
 	if (start) {
 		for (row = 0;
@@ -611,8 +678,9 @@ void window_page_up(struct view *view)
 void window_page_down(struct view *view)
 {
 	struct window *window = view->window;
-	unsigned start = screen_start(view);
-	unsigned row, bytes;
+	position_t start = screen_start(view);
+	int row;
+	size_t bytes;
 
 	for (row = 0; row + OVERLAP < window->rows; row++, start += bytes)
 		if (!(bytes = find_row_bytes(view, start, 0, window->columns)))
@@ -638,26 +706,27 @@ static void window_colors(void)
 	struct window *window, *w;
 
 	static rgba_t colors[][2] = {
-		{ 0x000000ff, ~0 },
+		{ DEFAULT_FGRGBA, DEFAULT_BGRGBA },
 		{ 0x00000000, 0x7f7f7f00 },
-		{ 0x0000ff00, 0x7f7f0000 },
-		{ 0xff00ff00, 0x007f0000 },
-		{ 0xffff0000, 0x00007f00 },
 		{ 0x00000000, 0xffffff00 },
 		{ 0x0000ff00, 0xffff0000 },
-		{ 0xff00ff00, 0x00ff0000 },
-		{ 0xffff0000, 0x0000ff00 },
+		{ 0x0000ff00, 0x7f7f0000 },
+		{ 0x00ff0000, 0xff00ff00 },
+		{ 0x00ff0000, 0x7f007f00 },
+		{ 0xff000000, 0x007f7f00 },
+		{ 0xff000000, 0x00ffff00 },
 		{ }
 	};
 
 	for (window = window_list; window; window = window->next)
 		window->bgrgba = 0;
-	active_window->fgrgba = 0xff;
-	active_window->bgrgba = ~0;
+	j = active_window != window_list || active_window->next;
+	active_window->fgrgba = colors[j][0];
+	active_window->bgrgba = colors[j][1];
 	for (window = window_list; window; window = window->next) {
 		if (window->bgrgba)
 			continue;
-		for (j = 0; colors[j][1]; j++) {
+		for (j = 1; colors[j][1]; j++) {
 			for (w = window_list; w; w = w->next)
 				if (w != window &&
 				    w->bgrgba == colors[j][1] &&
@@ -684,16 +753,21 @@ static void repaint(void)
 		       active_window->column + active_window->cursor_column);
 }
 
-int window_getch(void)
+Unicode_t window_getch(void)
 {
-	int block = 0;
+	Boolean_t block = FALSE;
 	for (;;) {
-		int ch = display_getch(display, block);
-		if (ch == DISPLAY_CHANGED)
+		Unicode_t ch = display_getch(display, block);
+		if (ch == ERROR_CHANGED)
 			window_raise(window_current_view());
-		else if (ch != DISPLAY_NONE)
+		else if (ch != ERROR_EMPTY)
 			return ch;
 		repaint();
-		block = 1;
+		block = TRUE;
 	}
 }
+
+unsigned window_columns(struct window *window)
+{
+	return window ? window->columns : 80;
+}
